function [f] = oxphos_dXdT(t,x,NADr,ADP_c)
%

%% define indices for reactants
iATP	=   1;
iADP    =   2;
iPi     =   3;
iNADH   =   4;
iNAD    =   5;
iQH2    =   6;
iCOQ    =   7;
iCox    =   8;
iCred   =   9;
iO2     =   10;
iH2O    =   11;

%%  LIST of MODEL PARAMETERS
%  (i) Thermochemical constants
RT = 8.314*(28+273.15)/1e3;     % kJ  mol^{-1}
% RT = 8.314*298.15/1e3;        % kJ  mol^{-1}
F = 0.096484;                   % kJ mol^{-1} mV^{-1}
%  (ii) Subcelular volumes and water spaces
Vmito = 0.056;                  % (ml mito / ml cell)
Vcyto = 0.894;                  % (ml cyto / ml cell)
W_m = 0.664*1.09;               % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;                  % Matrix water space
W_i = 0.1*W_m;                  % IM water space
%  (iii) Pooled concentrations
Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q + QH2
NADtot = 2.97e-3;               % M; total NAD+NADH
A_x    = 10e-3;                 % total ATP + ADP pool in matrix
A_c    = 8.2e-3;                % total ATP + ADP pool in cytoplasm
%  (iv) oxphos parameters
n_A = 3.0;  % unitless
k_O2   = 1.2e-4;
x_buff = 100;
%  (v) Outer membrane transport parameters
x_A    = 85;        % micron sec^{-1}
x_Pi2  = 327;       % micron sec^{-1}
gamma  = 5.99;      % mito membrane area per cell volume micron^{-1}

%  STATE VARIABLES
dPsi	= x(1);
QH2_x	= x(2);
Cred_i	= x(3);
ATP_x	= x(4);
Pi_x	= x(5);
ATP_i	= x(6);
ADP_i	= x(7);
Pi_i	= x(8);
H_x     = x(9); 
K_x     = x(10);
Mg_x    = x(11);

% Fixed concentrations
NAD_x = NADtot*NADr;
H_i   = 10^(-7.1);
Mg_i  = 1e-3;
K_i   = 150e-3;
Pi_c  = 1e-3;
O2    = 2.6e-5;

% Other concentrations computed from pooled variables:
NADH_x = NADtot - NAD_x;
ADP_x  = A_x - ATP_x;
ATP_c  = A_c - ADP_c;
COQ_x  = Qtot - QH2_x;
Cox_i  = Ctot - Cred_i;

% Dissociation constants for reatants ATP, ADP, PI
Kh(1) = 10^(-6.46); Km(1) = 10^(-3.58); Kk(1) = 10^(-1.75);
Kh(2) = 10^(-6.33); Km(2) = 10^(-2.61); Kk(2) = 10^(-1.44);
Kh(3) = 10^(-6.65); Km(3) = 10^(-1.58); Kk(3) = inf;

% compute binding polynomials for reactants ATP, ADP, PI
P_x(1) = 1 + H_x/Kh(1) + Mg_x/Km(1) + K_x/Kk(1);
P_x(2) = 1 + H_x/Kh(2) + Mg_x/Km(2) + K_x/Kk(2);
P_x(3) = 1 + H_x/Kh(3)  + Mg_x/Km(3);

P_i(1) = 1 + H_i/Kh(1) + Mg_i/Km(1) + K_i/Kk(1);
P_i(2) = 1 + H_i/Kh(2) + Mg_i/Km(2) + K_i/Kk(2);
P_i(3) = 1 + H_i/Kh(3)  + Mg_i/Km(3);

% ----------------------------------------------------

%% Flux expresssions in the oxidative phosphorylation 

%% substrate/ion transport
% Transport of ADP, ATP, and Pi across outer membrane:
J_ADP   = gamma*x_A*(ADP_c-ADP_i);
J_ATP   = gamma*x_A*(ATP_c-ATP_i);
J_Pi2   = gamma*x_Pi2*(Pi_c-Pi_i);    

%% -------------------------------
% 1. Complex I
% NADH_x + Q_x + 5H+_x <-> NAD+_x + QH2_x + 4H+_i + 4dPsi
x_C1 = 2.47e4;
% compute free energy of reaction from free energe of formation
dGr_C1o = -109.7;

% compute Keq from combined free energy of reactions (including potential
% change due to charge translocation)
Keq_C1 = exp(-(dGr_C1o + 4*F*dPsi)/RT);
% compute Kapp from Kapp = Keq*H_x^n/H_i^m*Product(Poly)  (n,m are
% stochi.coefficients)
Kapp_C1 = Keq_C1*H_x^5/H_i^4;
% Flux expression
J_C1 = x_C1*( Kapp_C1*NADH_x*COQ_x - NAD_x*QH2_x );

%% -------------------------------
% 2. Complex III
% QH2_x + 2cytoC(ox)3+_i + 2H+_x <-> Q_x + 2cytoC(red)2+_i + 4H+_i + 2dPsi
x_C3 = 0.665;
k_Pi3 = 28.1e-6;
k_Pi4 = 3.14e-3;
dGr_C3o = +46.69;
Keq_C3 = exp(-(dGr_C3o + 2*F*dPsi)/RT);
Kapp_C3 = Keq_C3*H_x^2/H_i^4;
% Flux expression
J_C3 = x_C3*((1+Pi_x/k_Pi3)/(1+Pi_x/k_Pi4))*...
     (Kapp_C3^0.5*Cox_i*sqrt(QH2_x) - Cred_i*sqrt(COQ_x) );

%% -------------------------------
% 3. Complex IV
% 2cytoC(red)2+_i + 0.5O2_x + 4H+_x <-> 2cytoC(ox)3+_x + H2O_x + 2H+_i +
% 2dPsi
x_C4 = 9.93e-5;
dGr_C4o = -202.2;
% 2 charges from translocation of proton, and the other 2 from cytoC
Keq_C4 = exp(-(dGr_C4o + 4*F*dPsi)/RT);
Kapp_C4 = Keq_C4*H_x^4/H_i^2;
% Flux expression
J_C4 = x_C4*(O2/(O2+k_O2))*exp(F*dPsi/RT)*(Cred_i/Ctot)*( Kapp_C4^0.5*Cred_i*(O2^0.25) - Cox_i );

%% -------------------------------
% 4. F1Fo-ATPase
% ADP3-_x + HPO42-_x + H+_x + n_A*H+_i <-> ATP4- + H2O + n_A*H+_x
x_F1 = 1000;
dGr_F1o = -4.51;
Keq_F1 = exp(-(dGr_F1o-n_A*F*dPsi)/RT);
Kapp_F1 = Keq_F1*H_i^n_A/H_x^(n_A-1)*P_x(1)/(P_x(2)*P_x(3));
% Flux expression
J_F1 = x_F1*(Kapp_F1*ADP_x*Pi_x - ATP_x);

%% -------------------------------
% 5. ANT
% ATP4-_x + ADP3-_i <-> ATP4-_i + ADP3-_x
x_ANT = 7.27e-3;
k_mADP = 3.6e-6;
ADP_i1 = ADP_i/P_i(2); % fADP_i
ATP_i1 = ATP_i/P_i(1); % fATP_i
ADP_x1 = ADP_x/P_x(2); % fADP_x
ATP_x1 = ATP_x/P_x(1); % fATP_x
theta = 0.60;
Psi_i = theta*dPsi;
Psi_x = (theta-1)*dPsi;
% Flux expression
J_ANT  = x_ANT*( ADP_i1/(ADP_i1+ATP_i1*exp(-F*Psi_i/RT)) - ADP_x1/(ADP_x1+ATP_x1*exp(-F*Psi_x/RT)) )*(ADP_i1/(ADP_i1+k_mADP));

%% -------------------------------
% 6. H+-Pi2 cotransporter
x_Pi1 = 2.0e7;
k_PiH = 1.0e-3;
H2PIi1 = Pi_i*(H_i/Kh(iPi))/P_i(3);
H2PIx1 = Pi_x*(H_x/Kh(iPi))/P_x(3);
% Flux expression
J_Pi1 = x_Pi1*(H_i*H2PIi1 - H_x*H2PIx1)/(H2PIi1+k_PiH);

%% -------------------------------
% 7. H+ leak
x_Hle = 250.0;
% Flux expression
J_Hle = x_Hle*dPsi*(H_i*exp(F*dPsi/RT)-H_x)/(exp(F*dPsi/RT)-1);

%% -------------------------------
% 8. K+/H+ anti-porter
x_KH = 5.65e+6;
% Flux expression
J_KH   = x_KH*( K_i*H_x - K_x*H_i );

%% ---------------------------------------------------
%% Computing time derivatives of state variables

f(1) = (1.48e5)*( 4*J_C1 + 2*J_C3 + 4*J_C4 - n_A*J_F1 - J_ANT - J_Hle);
f(2) = (J_C1 - J_C3)/W_x; 
f(3) = (2*J_C3 - 2*J_C4)/W_i; 
f(4) = (J_F1 - J_ANT)/W_x; 
f(5) = (-J_F1 + J_Pi1)/W_x; 
f(6) = (J_ATP + J_ANT)/W_i; 
f(7) = (J_ADP - J_ANT)/W_i;
f(8) = (-J_Pi1 + J_Pi2)/W_i;

% Concentrations of cation-binding reactants ATP, ADP, and PI
xx(1) = ATP_x;
xx(2) = ADP_x;
xx(3) = Pi_x;

% Time derivatives of cation-binding reactants
dxxdt(1) = +f(4);
dxxdt(2) = -f(4);
dxxdt(3) = +f(5);

% Partial Derivatives:
pHBpM = - sum( (H_x*xx./Kh)./(Km.*P_x.^2) );
pHBpK = - sum( (H_x*xx./Kh)./(Kk.*P_x.^2) );
pHBpH = + sum( (1+Mg_x./Km+K_x./Kk).*xx./(Kh.*P_x.^2) );
pMBpH = - sum( (Mg_x*xx./Km)./(Kh.*P_x.^2) );
pMBpK = - sum( (Mg_x*xx./Km)./(Kk.*P_x.^2) );
pMBpM = + sum( (1+H_x./Kh+K_x./Kk).*xx./(Km.*P_x.^2) );
pKBpH = - sum( (K_x*xx./Kk)./(Kh.*P_x.^2) );
pKBpM = - sum( (K_x*xx./Kk)./(Km.*P_x.^2) );
pKBpK = + sum( (1+H_x./Kh+Mg_x./Km).*xx./(Kk.*P_x.^2) );

% PHI's:
Phi_H = - sum( H_x*dxxdt./(Kh.*P_x) ) ...
        + (-(4+0)*J_C1 - (4-2)*J_C3 - (2+2)*J_C4 + (n_A-1)*J_F1 ...
           + 2*J_Pi1 + J_Hle - J_KH)/W_x;
Phi_M = - sum( Mg_x*dxxdt./(Km.*P_x) ) ;
Phi_K = - sum( K_x*dxxdt./(Kk.*P_x) ) + 1*J_KH/W_x ;

% alpha's:
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;

% Add additional buffer for [H+]
BX = 0.02; % M
K_BX = 1e-7; % M
aH = 1 + pHBpH + BX/K_BX/(1+H_x/K_BX)^2;

% Denominator:
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;

% Derivatives for H,Mg,K:
dH_xdt = ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
        
dMg_xdt = ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
        
dK_xdt = ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;      

f(9)  = dH_xdt;
f(10) = dK_xdt;
f(11) = dMg_xdt;

%% Output the time derivative matrix
f = f';

%% -------------------------------
% end of function
% --------------------------------
